# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import TYPE_CHECKING

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_raw_response_wrapper, async_to_raw_response_wrapper
from ..._base_client import make_request_options
from ...types.shared import (
    SharedSelfRecursion,
    SharedMutualRecursionA,
    SharedMutualRecursionB,
)

if TYPE_CHECKING:
    from ..._client import Sink, AsyncSink

__all__ = ["SharedResponses", "AsyncSharedResponses"]


class SharedResponses(SyncAPIResource):
    with_raw_response: SharedResponsesWithRawResponse

    def __init__(self, client: Sink) -> None:
        super().__init__(client)
        self.with_raw_response = SharedResponsesWithRawResponse(self)

    def create_mutual_a(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> SharedMutualRecursionA:
        return self._post(
            "/recursion/shared/responses/mutual-a",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=SharedMutualRecursionA,
        )

    def create_mutual_b(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> SharedMutualRecursionB:
        return self._post(
            "/recursion/shared/responses/mutual-b",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=SharedMutualRecursionB,
        )

    def create_self(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> SharedSelfRecursion:
        return self._post(
            "/recursion/shared/responses/self",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=SharedSelfRecursion,
        )


class AsyncSharedResponses(AsyncAPIResource):
    with_raw_response: AsyncSharedResponsesWithRawResponse

    def __init__(self, client: AsyncSink) -> None:
        super().__init__(client)
        self.with_raw_response = AsyncSharedResponsesWithRawResponse(self)

    async def create_mutual_a(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> SharedMutualRecursionA:
        return await self._post(
            "/recursion/shared/responses/mutual-a",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=SharedMutualRecursionA,
        )

    async def create_mutual_b(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> SharedMutualRecursionB:
        return await self._post(
            "/recursion/shared/responses/mutual-b",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=SharedMutualRecursionB,
        )

    async def create_self(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> SharedSelfRecursion:
        return await self._post(
            "/recursion/shared/responses/self",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=SharedSelfRecursion,
        )


class SharedResponsesWithRawResponse:
    def __init__(self, shared_responses: SharedResponses) -> None:
        self.create_mutual_a = to_raw_response_wrapper(
            shared_responses.create_mutual_a,
        )
        self.create_mutual_b = to_raw_response_wrapper(
            shared_responses.create_mutual_b,
        )
        self.create_self = to_raw_response_wrapper(
            shared_responses.create_self,
        )


class AsyncSharedResponsesWithRawResponse:
    def __init__(self, shared_responses: AsyncSharedResponses) -> None:
        self.create_mutual_a = async_to_raw_response_wrapper(
            shared_responses.create_mutual_a,
        )
        self.create_mutual_b = async_to_raw_response_wrapper(
            shared_responses.create_mutual_b,
        )
        self.create_self = async_to_raw_response_wrapper(
            shared_responses.create_self,
        )
