# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Optional, cast

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_raw_response_wrapper, async_to_raw_response_wrapper
from ..._base_client import make_request_options
from ...types.responses import (
    UnionTypeNumbersResponse,
    UnionTypeObjectsResponse,
    UnionTypeMixedTypesResponse,
    UnionTypeNullableUnionResponse,
    UnionTypeSuperMixedTypesResponse,
)

if TYPE_CHECKING:
    from ..._client import Sink, AsyncSink

__all__ = ["UnionTypes", "AsyncUnionTypes"]


class UnionTypes(SyncAPIResource):
    with_raw_response: UnionTypesWithRawResponse

    def __init__(self, client: Sink) -> None:
        super().__init__(client)
        self.with_raw_response = UnionTypesWithRawResponse(self)

    def mixed_types(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeMixedTypesResponse:
        """Endpoint with a top level union response of different types."""
        return cast(
            UnionTypeMixedTypesResponse,
            self._post(
                "/responses/unions/mixed_types",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeMixedTypesResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def nullable_union(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Optional[UnionTypeNullableUnionResponse]:
        """Endpoint with a top level union response of floats and integers."""
        return cast(
            Optional[UnionTypeNullableUnionResponse],
            self._post(
                "/responses/unions/nullable",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeNullableUnionResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def numbers(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeNumbersResponse:
        """Endpoint with a top level union response of floats and integers."""
        return cast(
            UnionTypeNumbersResponse,
            self._post(
                "/responses/unions/numbers",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeNumbersResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def objects(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeObjectsResponse:
        """Endpoint with a top level union response of just object variants."""
        return cast(
            UnionTypeObjectsResponse,
            self._post(
                "/responses/unions/objects",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeObjectsResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def super_mixed_types(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeSuperMixedTypesResponse:
        """Endpoint with a top level union response of different types."""
        return cast(
            UnionTypeSuperMixedTypesResponse,
            self._post(
                "/responses/unions/super_mixed_types",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeSuperMixedTypesResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncUnionTypes(AsyncAPIResource):
    with_raw_response: AsyncUnionTypesWithRawResponse

    def __init__(self, client: AsyncSink) -> None:
        super().__init__(client)
        self.with_raw_response = AsyncUnionTypesWithRawResponse(self)

    async def mixed_types(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeMixedTypesResponse:
        """Endpoint with a top level union response of different types."""
        return cast(
            UnionTypeMixedTypesResponse,
            await self._post(
                "/responses/unions/mixed_types",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeMixedTypesResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def nullable_union(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Optional[UnionTypeNullableUnionResponse]:
        """Endpoint with a top level union response of floats and integers."""
        return cast(
            Optional[UnionTypeNullableUnionResponse],
            await self._post(
                "/responses/unions/nullable",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeNullableUnionResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def numbers(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeNumbersResponse:
        """Endpoint with a top level union response of floats and integers."""
        return cast(
            UnionTypeNumbersResponse,
            await self._post(
                "/responses/unions/numbers",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeNumbersResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def objects(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeObjectsResponse:
        """Endpoint with a top level union response of just object variants."""
        return cast(
            UnionTypeObjectsResponse,
            await self._post(
                "/responses/unions/objects",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeObjectsResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def super_mixed_types(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> UnionTypeSuperMixedTypesResponse:
        """Endpoint with a top level union response of different types."""
        return cast(
            UnionTypeSuperMixedTypesResponse,
            await self._post(
                "/responses/unions/super_mixed_types",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    idempotency_key=idempotency_key,
                ),
                cast_to=cast(
                    Any, UnionTypeSuperMixedTypesResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class UnionTypesWithRawResponse:
    def __init__(self, union_types: UnionTypes) -> None:
        self.mixed_types = to_raw_response_wrapper(
            union_types.mixed_types,
        )
        self.nullable_union = to_raw_response_wrapper(
            union_types.nullable_union,
        )
        self.numbers = to_raw_response_wrapper(
            union_types.numbers,
        )
        self.objects = to_raw_response_wrapper(
            union_types.objects,
        )
        self.super_mixed_types = to_raw_response_wrapper(
            union_types.super_mixed_types,
        )


class AsyncUnionTypesWithRawResponse:
    def __init__(self, union_types: AsyncUnionTypes) -> None:
        self.mixed_types = async_to_raw_response_wrapper(
            union_types.mixed_types,
        )
        self.nullable_union = async_to_raw_response_wrapper(
            union_types.nullable_union,
        )
        self.numbers = async_to_raw_response_wrapper(
            union_types.numbers,
        )
        self.objects = async_to_raw_response_wrapper(
            union_types.objects,
        )
        self.super_mixed_types = async_to_raw_response_wrapper(
            union_types.super_mixed_types,
        )
