# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import TYPE_CHECKING

from ..types import Version1_30NameCreateResponse, version_1_30_name_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_raw_response_wrapper, async_to_raw_response_wrapper
from .._base_client import make_request_options

if TYPE_CHECKING:
    from .._client import Sink, AsyncSink

__all__ = ["Version1_30Names", "AsyncVersion1_30Names"]


class Version1_30Names(SyncAPIResource):
    with_raw_response: Version1_30NamesWithRawResponse

    def __init__(self, client: Sink) -> None:
        super().__init__(client)
        self.with_raw_response = Version1_30NamesWithRawResponse(self)

    def create(
        self,
        version_1_15: str | NotGiven = NOT_GIVEN,
        *,
        version_1_16: str | NotGiven = NOT_GIVEN,
        version_1_17: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Version1_30NameCreateResponse:
        """
        The `X-Client-Secret` header shouldn't be included in params definitions as it
        is already sent as a client argument.

        Whereas the `X-Custom-Endpoint-Header` should be included as it is only used
        here.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            f"/version_1_30_names/query/{version_1_15}",
            body=maybe_transform(
                {"version_1_17": version_1_17}, version_1_30_name_create_params.Version1_30NameCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
                query=maybe_transform(
                    {"version_1_16": version_1_16}, version_1_30_name_create_params.Version1_30NameCreateParams
                ),
            ),
            cast_to=Version1_30NameCreateResponse,
        )


class AsyncVersion1_30Names(AsyncAPIResource):
    with_raw_response: AsyncVersion1_30NamesWithRawResponse

    def __init__(self, client: AsyncSink) -> None:
        super().__init__(client)
        self.with_raw_response = AsyncVersion1_30NamesWithRawResponse(self)

    async def create(
        self,
        version_1_15: str | NotGiven = NOT_GIVEN,
        *,
        version_1_16: str | NotGiven = NOT_GIVEN,
        version_1_17: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Version1_30NameCreateResponse:
        """
        The `X-Client-Secret` header shouldn't be included in params definitions as it
        is already sent as a client argument.

        Whereas the `X-Custom-Endpoint-Header` should be included as it is only used
        here.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            f"/version_1_30_names/query/{version_1_15}",
            body=maybe_transform(
                {"version_1_17": version_1_17}, version_1_30_name_create_params.Version1_30NameCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
                query=maybe_transform(
                    {"version_1_16": version_1_16}, version_1_30_name_create_params.Version1_30NameCreateParams
                ),
            ),
            cast_to=Version1_30NameCreateResponse,
        )


class Version1_30NamesWithRawResponse:
    def __init__(self, version_1_30_names: Version1_30Names) -> None:
        self.create = to_raw_response_wrapper(
            version_1_30_names.create,
        )


class AsyncVersion1_30NamesWithRawResponse:
    def __init__(self, version_1_30_names: AsyncVersion1_30Names) -> None:
        self.create = async_to_raw_response_wrapper(
            version_1_30_names.create,
        )
